;;;
;;; basic.el (Not for BASIC)
;;;
;;; Created by S.Namba 
;;; Sun May  3 23:39:56 1992
;;;
;;; version 004d: Sun Jul 16 21:44:50 1995
;;;     Cleanup, fitting with V19.28 Emaxes, This is going to be the
;;;     final version of `BASIC' (I hope).
;;; version 004c: Wed Jan 26 11:59:49 1994
;;;	alias implemented, and line-move is alias for
;;;	next-line-internal in Emacs using next-line-internal (like 18.58, 18.59).
;;; version 004b: Sun Dec 5 18:47:49 1993, Wed Dec 15 15:05:23
;;;	 1993 line-move / next-line-internal support for 18.59
;;;	 Nemacs/Mule on linux (uses line-move function only in
;;;	 "18.55" or "19.19" is included in (emacs-version) output.
;;; version 004a: Mon Dec 28 21:50:27 1992
;;;	mule0.9.6 support (line-move / next-line-internal)
;;; version 004 : Thu Jun 18 18:39:49 1992
;;;	my-newline added.
;;;	kill-buffer-and-delete-window is added.
;;; version 003 : Sat May 30 14:12:29 1992
;;;	smooth-scroll is modified. bcc-err is modified.
;;;	bcc-compile can be called from program or
;;;	repeat-complex-command etc.
;;; version 002 : Sun May 10 19:25:24 1992
;;;	line-move is not used anymore. Used this-command (var)
;;;	faking in my-next-line, smooth-scroll-up/down.
;;; version 001 : Fri May  8 15:41:59 1992
;;;	Bug fix for my-delete-char (actually fix is in 
;;;	my-delete-backward-char).
;;;

;; For emacs 19.xx compatibility
;; You can use some 19.xx elisp library with this:
;;
;; Gee, alias faded away again (at least in 19.28 Emacs).
(or (fboundp 'alias)
    (fset 'alias 'fset))

;; If line-move is not fbounded, line-move is alias of next-line-internal.
(or (fboundp 'line-move)
    (alias 'line-move 'next-line-internal))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Modifying for my preference.
;;
;; Original next-line inserts \n if current line is the last
;; line in the buffer. I don't like this. If it should
;; insert \n, previous-line should also insert \n at the top
;; of buffer.
;;
;; So I provided my-next-line. I should have overridden
;; line-move in simple.el here, to handle goal-column's
;; correctly. I did this not override original next-line
;; because I don't want to affect other lisp library which
;; may depand on original feature of next-line...
;;
;; Four original routines in Nemacs to move or delete
;; horizontally do not work as you expect in Zenkaku
;; characters, if you give numeric argument (they do not
;; work by character-count but by byte-count).  So I added
;; my-forward-char, my-backward-char, my-delete-char and
;; my-backward-delete-char as shown below.
;;
;; You may want to modify bacward-delete-char-untabify in
;; lisp.el or j-bacward-delete-char in skk.el, to call these
;; four private routines.
;;
;; I once tried to override the four original routines, but
;; this method may affect all of the lisp library.  Thus I
;; decided to use private version, and modify lisp library
;; to call my private version only when I need it really
;; bad.  As a matter of fact, I patched
;; j-bacward-delete-char in skk.el, but I left
;; bacward-delete-char-untabify in lisp.el un-modified
;; because basically I don't use Japanese in .el files...
;;
;; Also I don't like behaviour of yank which sets
;; mark.  At least in my usage of emacs, this
;; implementation is harmful (All I want to do is cutting
;; and pasting lines with exchanging mark and point set at
;; two places in some distance in a text buffer. C-u C-@ is
;; annoying in this case).
;;
;; Further, I don't like the emacs feature that 'mark' is not
;; "window-local" but "buffer-local". Meaning that you can
;; not set mark at different position in two or more windows
;; displaying the same buffer content.  I am too lazy to
;; provide private mark object.
;; In MicroEMACS's, mark was "window-local"...
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;
;; line-move override was here, but deleted in v002.
;;

;;
;; This is neat and clean.
;; this-command is faked.
;;
(defun my-next-line (arg)
  "Move cursor vertically down ARG lines.
Almost the same with next-line, but never inserts newline in any case."
  (interactive "p")
  (line-move arg)
    (setq this-command 'next-line)
    nil)

;;
;; These two understand *BOTH* Zenkaku and Hankaku chars.
;; As many as numeric arg.  Exactly, man.
;;
(defun my-backward-char (n)
  "Move point left ARG characters (right if ARG negative).
On attempt to pass beginning or end of buffer, stop and signal error.
This version understands Zenkaku and Hankaku characters."
  (interactive "p")
  (let ((step (if (< n 0) -1 1))
	(count (if (< n 0) (- n) n)))
  (while (> count 0)
    (backward-char step)
    (setq count (1- count)))))

(defun my-forward-char (n)
  "Move point right ARG characters (left if ARG negative).
On reaching end of buffer, stop and signal error.
This version understands Zenkaku and Hankaku characters."
  (interactive "p")
  (my-backward-char (- n)))

;;
;; These two can erase *BOTH* Zenkaku and Hankaku chars.
;; As many as numeric arg.  Exactly, man.
;;
;; Do not set mark.
;;
;;    BUG. Sometimes 'Mark set' appears in echo area,
;;         even if it is not so.
;;

(defun my-delete-backward-char (n &optional killp)
  "Delete the previous ARG characters (following, with negative ARG).
Optional second arg KILLFLAG non-nil means kill instead (save in kill ring).
Interactively, ARG is the prefix arg, and KILLFLAG is set if
ARG was explicitly specified.
This version understands Zenkaku and Hankaku characters."
  (interactive "*p\nP")
  (let ((step (if (< n 0) -1 1))
	(count (if (< n 0) (- n) n)))
    (save-excursion			;; I don't want mark
      ;; Here was bug. Fri May  8 15:40:25 1992 s.n.
      ;; 'count' handling was wrong for call from delete-char.
      (if (or killp (> count 1) (< count -1))
	  ;; kill
	  (kill-region (point)
		       (save-excursion
			 (while (> count 0)
			   (backward-char step)
			   (setq count (1- count)))
			 (point)))
	;; else not kill...
	(delete-region (point)
		       (save-excursion
			 (while (> count 0)
			   (backward-char step)
			   (setq count (1- count)))
			 (point)))))))

(defun my-delete-char (n &optional killp)
  "Delete the following ARG characters (previous, with negative arg).
Optional second arg KILLFLAG non-nil means kill instead (save in kill ring).
Interactively, ARG is the prefix arg, and KILLFLAG is set if
ARG was explicitly specified.
This version understands Zenkaku and Hankaku characters."
  (interactive "*p\nP")
  (my-delete-backward-char (- n ) killp))

;;;
;;; Smooth scrolling inspired by MicroEMACS 3.9.
;;; Ancient idea seems to have came from DEC EDT...
;;; Tue Apr 28 21:19:57 1992
;;;
;;; This-command fake is used here as well.
;;; Sun May 10 19:33:19 1992
;;; Added code for re-calibration in case cursor is in peripheral area.
;;; Fri May 29 18:46:43 1992
;;;

(defun current-line-in-window  ()
  "Returns line count of current line in current window. No arg."
  (save-excursion
    ;; goto bol not to count \n at eol of this line...
    (beginning-of-line)
    (count-lines (point)
		 (save-excursion (move-to-window-line 0)
				 (point)))))

(defun smooth-scroll-up (arg)
  "Moves cursor down keeping cursor within 1/3 to 2/3 area of current window.
Scroll the window in need.
It's convenient to bind this command to down arrow key."
  (interactive "p")
  (let* ((height (1- (window-height)))
	 (parity (% height 2))
	 (top-edge (/ (1- height) 4))
	 (btm-edge (+ (1+ parity) (* top-edge 3)))
	 (c-line (current-line-in-window))
	 (bob-not-shown				;; var. to duck ding.
	  (not (pos-visible-in-window-p (point-min)))))
  ;; Old code.
  ;;	 (bob-not-shown (if (/= (point-min)	;; var. to duck ding.
  ;;			 (save-excursion
  ;;			   (move-to-window-line 0)
  ;;			   (point))) t nil)))

    (if
	;; Case cursor is within central area:
	(or (and (> arg 0)
		 (<= c-line btm-edge)
		 (< (- btm-edge c-line) arg))
	    (and (< arg 0)
		 (>= c-line top-edge)
		 ;;(< (- c-line top-edge) (- arg)) ;; old code for next line.
		 (> (- top-edge c-line)  arg)
		 bob-not-shown))
	(scroll-up arg)
      ;; else if cursor is in peripheral area:
      ;; re-calibrating cursor to central area.
      (progn
	(if (and (> arg 0)
		 (> c-line btm-edge))
	    (scroll-up (+ arg (- c-line btm-edge))))
	(if (and (< arg 0)
		 (< c-line top-edge)
		 bob-not-shown)
	    (scroll-up (+ arg (- c-line top-edge))))))
    (line-move arg)
    (setq this-command 'next-line)))

(defun smooth-scroll-down (arg)
  "Moves cursor up keeping cursor within 1/3 to 2/3 area of current window.
Scroll the window in need.
It's convenient to bind this command to up arrow key."
  (interactive "p")
  (smooth-scroll-up (- arg))
  (setq this-command 'previous-line))

;;; 
;;; my-newline, inspired from Freemacs.
;;; You don't need to fix skk.el.
;;; Thu Jun 18 17:25:58 1992
;;;

(defun my-newline (arg)
  "Moves cursor down keeping cursor within 2/3 area from the top of the current window.
Scroll the window in need."
  (interactive "p")
  (if (> arg 0)
      (progn
	(newline arg)
	(forward-line (- arg))
	(smooth-scroll-up arg))))

;;
;; I don't like the original yank because they set mark.
;; Even if you can 'pop' old mark by C-u C-@,
;; this popping is annoying when you should kill and yank repeatedly.
;;
;; I found kill-line does no harm (it doesn't set mark).
;(defun my-kill-line (n)
;  "Same as kill-line, but does not set mark."
;  (interactive "*P")		;; Must be "*P"
;  (save-excursion
;    (kill-line n)))

(defun my-yank (n)
  "Same as yank, but does not set mark."
  (interactive "*P")		;; Must be "*P"
  (let* (after-yank-point)
    (save-excursion
      (yank n)
      (setq after-yank-point (point)))
    (goto-char after-yank-point)))

;;
;; Enhancement of kill-buffer
;; Thu Jul 23 16:29:56 1992
;;
(defun kill-buffer-and-delete-window (arg &optional flag)
  "Replacement for original kill-buffer. Kill the its window too.
Kills BUFFER and delete its window,
if optional second ARG (prefix argument in interactive call).
Without optional second arg, leaves the window undeleted (the
same operation as original kill-buffer)."
  (interactive "bKill buffer (and delete window with prefix arg): \nP")
  (let ((buf (if arg (get-buffer arg) (current-buffer))))
    (if (and (interactive-p)
	     (buffer-file-name)
	     (buffer-modified-p buf))
	(if (yes-or-no-p (format "Buffer %s is modified; Kill it anyway? "
				 (buffer-name buf)))
	    (progn (set-buffer buf)
		   (set-buffer-modified-p nil)
		   (if flag
		       (delete-windows-on buf))
		   (kill-buffer buf)))
      	  ;; If user did not say 'yes', do nothing.
      (progn (if flag
		 (delete-windows-on buf))
	     (kill-buffer buf)))))

;;
;; Marking sentenses.
;; "from the point" style and "mark-paragraph" style.
;; Mon Aug  3 15:51:21 1992
;;
(defun mark-sentence (arg)
  "Set mark ARG sentense from point."
  (interactive "p")
  (push-mark
    (save-excursion
      (forward-sentence arg)
      (point))))

(defun mark-whole-sentence ()
  "Put point at beginning of this paragraph, mark at end."
  (interactive)
  (forward-sentence 1)
  (push-mark nil t)
  (backward-sentence 1))

;;;;;;;;;;;;;;;;;;;;;;
;;; Original routines
;;;;;;;;;;;;;;;;;;;;;;

;;;
;;; Preprocess for BCC/BCCX (Borland C++) error
;;; message for next-error command.
;;; Very rough and ready.
;;; Thu Apr 23 15:04:43 1992
;;;
(if (eq system-type 'ms-dos)
    (progn (defun bcc-err ()
	     "Preprocess brain-damaged 'bcc' error message for (next-error) command.
Replaces the space before the line number with colon.
Very rough and ready because:

All '\\' in error message are replaced with '/'.
Just .c and .h files are processed.
C++ source file is ignored.
Sorry.  I miss good old TCC..."
	     (interactive)
	     (replace-string "\\" "/")
	     (goto-char (point-min))
	     (replace-regexp "^\\(.+\\.[ch]\\) " "\\1:"))

	   (defun bcc-compile (command)
	     "Call (compile) and then (bcc-err)."
	     (interactive (list (read-string "Compile command: " compile-command)))
	     (compile command)
	     (set-buffer "*compilation*")
	     (goto-char (point-min))
		  (bcc-err))))

;;
;; My timestamp.
;;
(defun insert-current-time-string ()
"Inserts current time string at point."
  (interactive)
  (insert (current-time-string)))

;;
;; Reverse operation of C-x o (other-window).
;;
(defun another-window (arg)
  "Select the ARG'th different window in inverse direction."
  (interactive "p")
  (other-window (- arg)))

;;
;; Reverse operation of C-M-v (scroll-other-window)
;; '- for arg to scroll-other-window means full window scroll.
;;
(defun scroll-backward-other-window (arg)
  "Scroll next window backward ARG lines; or near full screen if no ARG.
The next window is the one below the current one; or the one at the top
if the current one is at the bottom.
When calling from a program, supply a number as argument or nil."
  (interactive "P")
  (scroll-other-window (if arg (- (prefix-numeric-value arg)) '-)))
