;; *CAUTION*
;; This program does not work with version 19.xx Emacs.
;; *CAUTION*

;;
;; dired-z.el, A Return Of A Son Of A Enhancement For Dired.
;; Author S.Namba (GBC00441@nifty.serve.or.jp
;; Assumed to be treated this file as absolute PDS.
;; No warrantedy.
;; Fri Jun 24 17:46:16 1994
;;

;;
;; Sometimes we should browse a lot of compressed or/and tar'ed files
;; in CDROM these days.  In normal dired (even if it is 'enhanced'
;; version), you'll get tired of issuing "!zcat" or "!tar zvtf"
;; commands after all the browsing session.  This package provides t
;; (dired-mark-tar), T (dired-mark-lha), and Z (dired-mark-zcat)
;; commands to sooth your pain.  I (dired-mark-mandoc) is provided.
;;

;; Assumes 'enhanced' dired (disclosed in Demacs or 'diredall' package)
;; is installed in your load-path.

;;
;; Place this file (or compressed one) in your load-path.
;; Load this file (or compressed one) from your ~/.emacs file as:
;;    (load "dired-hook")
;; In doing this, simply substitute (auto)loading line in your
;;  ~/.emacs (~/_emacs in DOS) with above line.
;;

;; *CAUTION*
;; No matter how you use this packages or not, customizations for
;; normal dired should be done after loading dired, or with
;; dired-load-hook, since some variables in dired.el, or direddos.el
;; are initialized by 'setq' function (this causes lots of pain).
;; This package directly loads dired, so your customizations lines
;; should be placed after the (load "dired-hook") in your .emacs file,
;; or place dired-load-hook initialization line before the (load
;; "dired-hook").
;; *CAUTION*

;;
;; In non-MSDOS system, programs like 'tar', 'lha' or 'zcat' can be
;; executed  on background if you invoked t (dired-mark-tar), T
;; (dired-mark-lha), or Z (dired-mark-zcat) command etc. 
;; If  you want to run them asynchronously, put next line in
;; your .emacs file:
;;    (setq dired-archivers-work-background t)
;; This requires 'background' package included in 'cmushell' package.
;;

;;
;; The default MSDOS 'tar' command for this package is 'kmtar.exe'.
;; You'd better get kmtar.exe, if you don't have one. kmtar.exe is 
;; a soup-up'ed version of tar.exe which can handle compressed tar
;; archive (decompression is automatically done without 'z' flag).
;; Actually 'kmtar' package contains a binary under name of 'tar.exe',
;; but I renamed it as 'kmtar.exe' because my old default is GNU
;; tar.exe.  So this package uses 'kmtar' for command name of MSDOS
;; tar.  In anyway, if the name of your MSDOS 'tar' command is not
;; 'kmtar.exe', put next line in your .emacs file (in this case
;; 'pdstar.exe'): (setq dired-tar-command "pdstar") If your DOS tar
;; command is kmtar under the name 'kmtar.exe' you don't need this
;; customization.
;;

;;
;; In default, tar files are assumed to be compressed (or gzip'ed),
;; because I believe you already have 'tar-mode.el' for browsing 
;; un-compressed tar archive.  However, if most of your tar files
;; are not compressed, you may want to put
;; next line in your .emacs file:
;;    (setq dired-tar-option-prefix "-v")
;;
;; If your DOS tar command is 'kmtar.exe' you don't need this
;; customization.  You can add any preferable option character in this
;; string.  For example, you can put block factor option for an
;; archive processed with some weired tar's.
;;    (setq dired-tar-option-prefix "-b 1 -v")
;;

;;
;; This package uses redirection/pipe facility.
;; Old Demacs 1.2.0 can't handle redirection or pipe facility for subprocess.
;; If you are using vanilla Demacs 1.2.0, patch it with attached diff.
;;
(load "emacs-19")
(setq dired-mode-map nil)
;; In anyway, you will need dired.  So I do load it here.
(load "dired")
;; And this customization is done with ease:
(define-key dired-mode-map "Z" 'dired-mark-zcat)
(define-key dired-mode-map "T" 'dired-mark-lha)
(define-key dired-mode-map "t" 'dired-mark-tar)
(define-key dired-mode-map "I" 'dired-mark-mandoc)

(defvar dired-tar-command (if (eq system-type 'ms-dos) "kmtar" "tar")
  "tar command string.")

(defvar dired-tar-option-prefix (if (eq system-type 'ms-dos) "v" "vz")
  "'z' flag for 'compress' option for GNU tar. 'kmtar' in DOS needs none.")
  
(defvar dired-archivers-work-background nil
  "If non-nil, programs like 'tar', 'lha' or 'zcat' work on background.")

(defun dired-mark-zcat ()
  "Invokes zcat for marked files."
  (interactive)
  (let ((fns (mapconcat (function shell-quote)
			 (dired-mark-get-files t t nil)
			 dired-mark-separator)))
    (if (> dired-mark-count 1)
	(setq fns (concat dired-mark-prefix fns dired-mark-postfix)))
    (dired-run-shell-command (concat "zcat " fns)
			     nil)))

(defun dired-mark-lha (extract)
  "Invokes 'lha' to produce archive list for marked files. Extracts content if with ARG." 
  (interactive "P")
  (let ((fns (mapconcat (function shell-quote)
			 (dired-mark-get-files t t nil)
			 dired-mark-separator)))
    (if (> dired-mark-count 1)
	(setq fns (concat dired-mark-prefix fns dired-mark-postfix)))
    (dired-run-shell-command
     (if (eq system-type 'ms-dos) 
	 (format
	  "for %%i in (%s) do lha %s %%i <con" fns
	  (if extract "x " "v "))
;	 (concat
;	  "for %i in (" fns ") do lha " (if extract "x " "v ") "%i" "< con")
       (format
	"for i in %s ; do echo Files in $i\\:; lha %s $i; done" fns
	(if extract "x " "v ")))
     dired-archivers-work-background)))

(defun dired-mark-tar (extract)
  "Invokes 'tar' to produce a file list for compressed tar file. Extracts content if with ARG." 
  (interactive "P")
  (let ((fns (mapconcat (function shell-quote)
			 (dired-mark-get-files t t nil)
			 dired-mark-separator)))
    (if (> dired-mark-count 1)
	(setq fns (concat dired-mark-prefix fns dired-mark-postfix)))
    (dired-run-shell-command
     (if (eq system-type 'ms-dos) 
	 (format "for %%i in (%s) do %s %s%s %%i" fns
		 dired-tar-command dired-tar-option-prefix
		 (if extract "xf " "tf "))
;	 (concat
;	  "for %i in (" fns ") do " dired-tar-command " "
;	  dired-tar-option-prefix
;	  (if extract "xf " "tf ")
;	   "%i")
       (format "for i in %s ; do echo Files in $i\\:; %s %s%s $i; done"
	       fns dired-tar-command dired-tar-option-prefix
	       (if extract "xf " "tf ")))
     dired-archivers-work-background)))

(defvar dired-manpage-formatter "groff -Tascii -mandoc -"
  "Your manpage formatter name.")

(defun dired-mark-mandoc (arg)
  "Marked all 'man' files are formatted. With ARG, the files are pre-zcat'ed.
This command is not executed on background."
  (interactive "P")
  (let (fns)
    (setq fns (mapconcat (function shell-quote)
			 (dired-mark-get-files t t nil)
			 dired-mark-separator))
    (dired-run-shell-command
     (concat (if arg "z" "")
	     "cat " fns  "| "  dired-manpage-formatter " | col -xbf")
     nil)))

(defun kill-current-buffer ()
  (interactive)
  (kill-buffer (current-buffer)))

(or (eq system-type 'ms-dos)
    (require 'background))
