;; Buffer menu function.
(setq buffer-menu-mode-hook
      '(lambda ()
	 (define-key Buffer-menu-mode-map "g" 'list-buffers)
	 (define-key Buffer-menu-mode-map "%" 'Buffer-menu-toggle-read-only)))

(defun Buffer-menu-toggle-read-only ()
  "Toggle read only flag of buffer on this line."
  (interactive)
  (save-excursion
    (set-buffer (Buffer-menu-buffer t))
    (toggle-read-only))
  (save-excursion
   (beginning-of-line)
   (forward-char 2)
   (let ((buffer-read-only nil))
     (if (looking-at "%")
	 (progn (delete-char 1)
		(insert ? ))
       (delete-char 1)
       (insert ?%)))))

;; copy or move region to at latest pushed mark.

(defun copy-region-pop-mark-and-yank (rb re)
  "Copies current region by copy-region-as-kill and yank it at popped mark.
You set mark to the yank point first, then make prefered region, and call this."
  (interactive "r")
  (copy-region-as-kill rb re)
  (set-mark-command t)
  (exchange-point-and-mark)
  (yank))

(defun kill-region-pop-mark-and-yank (rb re)
  "Region moving. Kills current region by kill-region and yank it at popped mark. 
You set mark to the yank point first, then make prefered region, and call this."
  (interactive "r")
  (kill-region rb re)
  (set-mark-command t)
  (exchange-point-and-mark)
  (yank))

;; No binding for kill-region-pop-mark-and-yank.
(global-set-key "\eY" 'copy-region-pop-mark-and-yank)

;;
;; 'wc' for current buffer.
;; Thu Oct  1 17:28:40 1992
;;
(defun count-word-region (rbeg rend)
  "The 'word' syntax is the same as forward-word command.
This may be different from that of unix 'wc' command.
Two ARGS are the region start and region end.
Returns the word count.
S.Namba, Thu Oct  1 17:03:01 1992"
  (interactive "r")
  (save-excursion
    (goto-char rbeg)
    (let ((wcount 0)
	  (ending (1- rend)))  ;; This ducks \n-in-EOB bug.
      (while (< (point) ending)
	(forward-word 1)
	(setq wcount (1+ wcount)))
      (message "%d words there." wcount)
      wcount)))

(defun count-word-buffer ()
  "The 'word' syntax is the same as forward-word command.
This may be different from that of unix 'wc' command.
Returns count of the word in the buffer.
S.Namba, Thu Oct  1 17:03:01 1992"
  (interactive)
  (count-word-region (point-min) (point-max)))

;;
;; Simple calc mode (adding only).
;; S.Namba, Tue Oct 13 16:35:57 1992
;;
(defun add-region (rb re &optional arg)
  "Add lines as numeric data in region. Result is inserted if called with ARG.
Very primitive calc mode."
  (interactive "r\nP")
  (let ((result (add-region1 rb re)))
    (if arg (if buffer-read-only
		(barf-if-buffer-read-only)
	      (progn (end-of-line 1) (newline 1)
		     (insert (int-to-string result))))
      (message "%d" result))))

(defun add-region1 (rb re)
  "Used by add-region."
  (save-excursion
    (let ((result 0))
      (goto-char rb)
      (while (< (point) re)
	(setq result (+ result (string-to-int
		   (buffer-substring
		    (save-excursion (beginning-of-line 1) (point))
		    (save-excursion (end-of-line 1) (point))))))
	(forward-line 1))
      result)))

(defun add-rectangle (rb re &optional arg)
  "Add lines as numeric data in rectangle. Result is inserted if called with ARG.
Very primitive calc mode."
  (interactive "r\nP")
  (let ((work (get-buffer-create " *Add-work*"))
	(lines (extract-rectangle rb re))
	result)
    (save-excursion
      (set-buffer work)
      (erase-buffer)
      (while lines
	(insert (car lines) ?\n)
	(setq lines (cdr lines)))
      (setq result (add-region1 (point-min) (point-max))))
    (if arg (if buffer-read-only
		(barf-if-buffer-read-only)
	      (progn
		(end-of-line 1) (newline 1)
		(insert (int-to-string result))))
      (message "%d" result))))
