;;
;; face-re-search.el V4.2
;;
;;   A `face' version of re-search-\(for\|back\)ward
;;   S.Namba <GBC00441@niftyserve.or.jp> (c)1995,1996 (but, copy freely)
;;
;; V2: the one I posted (7/4/95) to fj.editor.mule, the first public exposure.
;; V3: the thing posted (7/6/95) to fj.editor.mule.
;;     (query-replace "delete-face-re-search" "delete-re-search-face" nil),
;;     repaint face according to hilit-auto-rehighlight to make it faster.
;;     delete-face-re-search is not `interactive' anymore.
;; V4: Now works fine in non-X environment as well.
;; V4.1: Interactive facing interface added. Some func barfs on non-X yet.
;; V4.2: Use 'bold-italic and 'underline each for re-search-face and my-face.

;; Add this sort of key bindings in your .emacs if you like:
;;    (global-set-key "\e\C-s" 'face-re-search-forward)
;;    (global-set-key "\e\C-r" 'face-re-search-backward)
;;   or
;;    (global-set-key "\eS" 'face-re-search-forward)  ;; means M-shift-s
;;    (global-set-key "\eR" 'face-re-search-backward) ;; means M-shift-r
;; *AND* these autoloading codes (*required*).
;;    (autoload 'face-re-search-forward "face-re-search" nil t)
;;    (autoload 'face-re-search-backward "face-re-search" nil t)
;;    (autoload 'face-region "face-re-search" nil t)
;;    (autoload 'unface-region "face-re-search" nil t)

(if window-system (require 'hilit19))
(make-face 're-search-face)
(make-face 'my-face)

;; Set your favourite property here.
;;(if window-system
;;    (set-face-background 're-search-face "blue")
;;  (copy-face 'underline 're-search-face))

(defun face-re-search-reset ()
  "Initializer for face-re-search."
  (if window-system
      (set-face-background 're-search-face "blue")
    (copy-face 'underline 're-search-face))
  (if window-system
      (set-face-background 'my-face "hotpink")
    (copy-face 'bold-italic 'my-face)))

(face-re-search-reset) ;; Initialize here.

(defun face-re-search-forward (regexp)
  "Search forward from point for regular expression REGEXP, and set
special face 're-search-face for sought expression.  If REGEXP is null
string, clears 're-search-face already set in current buffer.
Facing is always done."
  (interactive "sface RE search: ")
  (delete-re-search-face (point-min) (point-max))
  (if (string-equal "" regexp)
      ()
    (let ((pt (re-search-forward regexp (point-max) t)))
      (face-re-search-region (point-min) (point-max) regexp)
      (if pt (goto-char pt)))))

(defun face-re-search-backward (regexp)
  "Search backward from point for match for regular expression REGEXP,
and set special face 're-search-face for sought expression.  If REGEXP
is null string, clears 're-search-face already set in current buffer.
Facing is always done."
  (interactive "sface RE search backward: ")
  (delete-re-search-face (point-min) (point-max))
  (if (string-equal "" regexp)
      ()
    (let ((pt (re-search-backward regexp (point-min) t)))
      (face-re-search-region (point-min) (point-max) regexp)
      (if pt (goto-char pt)))))

(defun face-re-search-region (rb re regexp)
  (save-excursion
    (goto-char rb)
    (while (re-search-forward regexp re t)
      (overlay-put (make-overlay (match-beginning 0) (match-end 0))
       'face 're-search-face))
    (run-hooks 'face-re-search-hook)))

;;
;; I referred to
;;       (hilit-rehighlight-buffer), 
;;       (hilit-unhighlight-region start end  t),
;;       (hilit-highlight-region   start end t),
;;       (hilit-repaint-command)
;; in hilit19.el.
;;
(defun delete-re-search-face (rb re)
  "Deletes re-search-face in REGBEG REGEND."
  (delete-face 're-search-face rb re))
  
(defun delete-face (face rb re)
  "Deletes FACE in REGBEG REGEND."
  ;; Remove other face. This must be done on selected 
  ;; region (or whole buffer, when called from face-re-search-*.
  ;; The search code puts 're-search-face for whole buffer).
  (if window-system (hilit-unhighlight-region rb re t))
  ;; Main
  (save-excursion
    (goto-char rb)
    (while (< (point)  re)
      (goto-char (next-overlay-change (point)))
      (let ((ov (car (overlays-at (point)))))
	(if (and ov (eq face (overlay-get ov 'face)))
	    (delete-overlay ov)))))
  ;; Repaint according to hilit-auto-rehighlight. Makes it little
  ;; little bit faster. Smugged things from hilit-repaint-command.
  (if window-system
      (let ((st) (en))
	(cond ((or (eq  hilit-auto-rehighlight 'visible) 
		   (eq hilit-auto-rehighlight '-))
	       (setq st (window-start) en (window-end) quietly t))
	      ((numberp hilit-auto-rehighlight)
	       (setq
		st (save-excursion
		     (forward-line (- hilit-auto-rehighlight)) (point))
		en (save-excursion
		     (forward-line hilit-auto-rehighlight) (point))))
	      (hilit-auto-rehighlight (setq st rb en re)))
	(if window-system (hilit-highlight-region st en nil t)))))

;;
;; Interactive commands added from V4.1
;; 
;;(if window-system (require 'hilit19)) ;; Required above.
;;(make-face 'my-face)

;; Set your favourite property here.
;;(if window-system
;;    (set-face-background 'my-face "hotpink")
;;  (copy-face 'bold-italic 'my-face))

(defun face-region (rb re)
  (interactive "r")
  (overlay-put (make-overlay rb re)
       'face 'my-face))

(defun unface-region (rb re)
  (interactive "r")
  (delete-face
   'my-face
   ;; It detects border !
   (if (= (point-min) rb)
       (point-min) (1- rb))
   re))

(defun next-face ()
  (interactive)
  (let ((pt (next-overlay-change (point))))
    (if (= pt (point-max))
	(error "next-face not found.")
      (goto-char pt)
      (while (looking-at "[ \t\n]")
	(forward-char 1) ))))

;; Experimental
(defun previous-face ()
  (interactive)
  (let ((pt (next-overlay-change (point))))
    (backward-char 1)
    (while (= pt (next-overlay-change (point)))
      (backward-char 1))))

;; Sun Mar  7 18:42:33 1999
;; Mon Mar  8 17:15:17 1999 (Hook suicides)
;; Support mule-2.3(19.34).
;; I don't know why face-re-search-region() requires face-re-search-reset(),
;; while face-region don't.
(if (and (string< "19.29" emacs-version)
	 (string< emacs-version "20"))
    (add-hook 'face-re-search-hook 'face-re-search-onetime-init-workaround))

(defun face-re-search-onetime-init-workaround ()
  "Workaround for mule-2.3\(19.34\). Do init, then remove itself from face-re-search-hook."
  (face-re-search-reset)
  (remove-hook				;; suicide.
   'face-re-search-hook 'face-re-search-onetime-init-workaround))

(provide 'face-re-search)
