;;
;; invisible.el:
;;	defines C-XB (list-invisible-buffers)
;;
;; Fri Aug  5 21:15:36 1994
;;
(defun get-invisible-buffer-list ()
  "Returns a list of invisible buffer whose name starts with a space."
  (let ((the-list (buffer-list))
	(inv-bufs nil) buf)
    (while the-list
      (setq buf (car the-list))
      (if (string-equal " " (substring (buffer-name buf) 0 1))
	  (setq inv-bufs (append inv-bufs (list buf))))
      (setq the-list (cdr the-list)))
    inv-bufs))

(defun get-buffer-data (buf)
  "Returns BUFFER's specs in buffer-menu format."
  (save-excursion
    (set-buffer buf)
    (format " %s%s %-13s  %-5s %-14s %s\n"
	    (if (buffer-modified-p) "*" " ")
	    (if buffer-read-only "%" " ")
	    (buffer-name)
	    (buffer-size)
	    mode-name
	    (buffer-file-name))))

(defun list-invisible-buffers (arg)
  "Similar to list-buffers, but also lists invisible buffers.
Calling with ARG means mention only file buffers, except 
if ARG is '-', only invisible buffers are listed."
  (interactive "P")
    (list-buffers arg)
    (save-excursion
      (set-buffer "*Buffer List*")
      (let ((buffer-read-only nil)
	    (invisibles (get-invisible-buffer-list)))
	(if (eq arg '-)
	    (progn (goto-line 3) (beginning-of-line nil)
		   (delete-region (point) (point-max))))
	(goto-char (point-max))
	(while invisibles
	  (insert (get-buffer-data (car invisibles)))
	  (setq invisibles (cdr invisibles))))))

(global-set-key "\C-xB" 'list-invisible-buffers)


