;;;
;;; microem.el
;;;
;;; Created by S.Namba 
;;;
;;; last-burried-buffer (bury-buffer), switch-to-alternate-buffer etc.
;;; Last update: Mon May 15 16:18:39 1995
;;;

;; bury-buffer is next-buffer in MicroEmacs 3.9.
;; C-xx is for compatibility to MicroEmacs 3.9.
(defun last-burried-buffer ()
  "Reverse movement of burry-buffer.
Rather slow becase invisible buffer visit is omitted."
  (interactive)
  (let ((the-list (buffer-list)) dest-name)
    (while (equal
	    " " (substring
		 (setq dest-name
		       (buffer-name
			(car (last-cell the-list)))) 0 1))
      (setq the-list (omit-last-cell the-list)))
    (switch-to-buffer dest-name)))

(defun last-cell (lis)
  "Body of last-burried-buffer. Returns a list containing the last cell only."
  (let ((tem (cdr lis)))
    (if (cdr tem) (last-cell tem) tem)))

(defun omit-last-cell (xlist)
  "Literal as its name!"
  (let ((last (cdr xlist)))
    (if last 
	(cons  (car xlist) (omit-last-cell last))
      nil)))

;; Ugly ol' things
;; old name: buffer-in-the-bottom
;(defun last-cell (b-list)
;  "Body of last-burried-buffer. Returns a list containing the last cell only."
;  (let* ((xn-1  (cdr b-list))	;; should be let* !
;	 (xn (cdr xn-1)))
;    (if (not (eq xn nil))
;	(last-cell xn-1)
;      xn-1)))
;
;(defun omit-last-cell-in-list (xlist)
;  "Literal as its name!"
;  (let ((last (cdr xlist)))
;    (if (eq nil last)
;	nil
;      (cons  (car xlist) (omit-last-cell-in-list last)))))

;;
;; Sometimes C-x b RET is annoying, so...
;;
(defun switch-to-alternate-buffer ()
  "Compatible to C-x b (switch-to-buffer) RET."
  (interactive)
  (let ((the-list (cdr (buffer-list))) dest-name disp-win)
    ;; Omit System's buffer, current-buffer and already displayed buffer
    (while (and the-list
		(or 
		 (string-equal " "
			(substring
			 (setq dest-name (buffer-name (car the-list))) 0 1))
		 ;; Save displayed buffer's window.
		 (setq disp-win (get-buffer-window (get-buffer dest-name)))
		 ;; This may not be needed (cdr (buffer-list))
		 (eq (current-buffer) (get-buffer dest-name))))
      (setq the-list (cdr the-list)))
    (if the-list
	(setq dest-name (buffer-name (car the-list)))
      ;; case no buffer to switch,
      (if (one-window-p)
	  (progn (get-buffer-create "*scratch*")
		 (save-excursion (set-buffer "*scratch*")
				 (lisp-interaction-mode))
		 (setq dest-name "*scratch*"))
	(setq dest-name (buffer-name (window-buffer disp-win)))))
    ;;(message "%s"  dest-name)
    ;; Don't switch to minibuffer !
    (or (string-match " \\*Minibuf" dest-name)
	(switch-to-buffer dest-name))))

(global-set-key "\C-xx"		'bury-buffer)
;;(global-set-key "\C-x\C-y"	'bury-buffer)
(global-set-key "\C-xy"		'last-burried-buffer)
(global-set-key "\C-xX"		'copy-to-register)

(if  (< (string-to-int emacs-version) 19)
    (global-set-key "\M-o" 'switch-to-alternate-buffer))

; Ue like bindings.
;(global-set-key "\^z"		'scroll-down)
;(global-set-key "\e\^g"	'goto-line)
;(global-set-key "\C-xz"	'enlarge-window)
;(global-set-key "\C-x\C-z"	'shrink-window)
;(global-set-key "\C-xc"	'suspend-emacs)
; Ue like bindings.
