;;
;; For emacs-20.x
;; Lot of vi/less facilities are supported in emacs-20.2's view.el.
;; Simpler than viewhook.el (for emacs-19).
;;

(global-set-key "\C-xV"		'my-view-buffer)
(defun my-view-buffer ()
  "Sets view-mode to current-buffer."
  (interactive)
  (view-buffer (current-buffer)))

;; From emacs-20.2's view.el (commented out in the original)
(defun View-goto-line-last (&optional line)
"Move to last (or prefix LINE) line in View mode.
Display is centered at LINE.
Sets mark at starting position and pushes mark ring."
  (interactive "P")
  (push-mark)
  (if line (goto-line (prefix-numeric-value line))
    (goto-char (point-max))
    (beginning-of-line))
  (view-recenter))

(add-hook
 'view-mode-hook
 '(lambda ()
    (progn
      (define-key view-mode-map "v" 'View-exit)
      (define-key view-mode-map "N" 'View-search-last-regexp-backward)
      (define-key view-mode-map "?" 'View-search-regexp-backward) ; Less does this.
      (define-key view-mode-map "G" 'View-goto-line-last)
      (define-key view-mode-map "k" 'View-scroll-line-backward)
      (define-key view-mode-map "j" 'View-scroll-line-forward)
      (define-key view-mode-map "b" 'View-scroll-page-backward)
      (define-key view-mode-map "f" 'View-scroll-page-forward)
      )))

(if (string< emacs-version "19.29") ;; in 19.34, 20.2 use ~/elisp/view20.el(c)
    ()
  ;; Autoload not working (why).
  ;; So load view20 => view20.bare (like viewhook.el).
  ;;(autoload 'view-buffer "view20" nil t)
  ;;(autoload 'view-file   "view20" nil t)
  ;; load view-20
  (require 'view20)
  (progn
    (define-key view-mode-map "C" 'View-kill-and-leave)
    (define-key view-mode-map "c" 'View-leave)
    (define-key view-mode-map "Q" 'View-quit-all)
    (define-key view-mode-map "E" 'View-exit-and-edit)
    ;;    (define-key view-mode-map "v" 'View-exit)
    (define-key view-mode-map "e" 'View-exit)
    (define-key view-mode-map "q" 'View-quit)
    ;;    (define-key view-mode-map "N" 'View-search-last-regexp-backward)
    (define-key view-mode-map "p" 'View-search-last-regexp-backward)
    (define-key view-mode-map "n" 'View-search-last-regexp-forward)
    ;;    (define-key view-mode-map "?" 'View-search-regexp-backward) ; Less does this.
    (define-key view-mode-map "\\" 'View-search-regexp-backward)
    (define-key view-mode-map "/" 'View-search-regexp-forward)
    (define-key view-mode-map "r" 'isearch-backward)
    (define-key view-mode-map "s" 'isearch-forward)
    (define-key view-mode-map "m" 'point-to-register)
    (define-key view-mode-map "'" 'register-to-point)
    (define-key view-mode-map "x" 'exchange-point-and-mark)
    (define-key view-mode-map "@" 'View-back-to-mark)  
    (define-key view-mode-map "." 'set-mark-command)
    (define-key view-mode-map "%" 'View-goto-percent)
    ;;    (define-key view-mode-map "G" 'View-goto-line-last)
    (define-key view-mode-map "g" 'View-goto-line)
    (define-key view-mode-map "=" 'what-line)
    (define-key view-mode-map "F" 'View-revert-buffer-scroll-page-forward)
    ;;    (define-key view-mode-map "k" 'View-scroll-line-backward)
    (define-key view-mode-map "y" 'View-scroll-line-backward)
    ;;    (define-key view-mode-map "j" 'View-scroll-line-forward)
    (define-key view-mode-map "\n" 'View-scroll-line-forward)
    (define-key view-mode-map "\r" 'View-scroll-line-forward)
    (define-key view-mode-map "u" 'View-scroll-half-page-backward)
    (define-key view-mode-map "d" 'View-scroll-half-page-forward)
    (define-key view-mode-map "z" 'View-scroll-page-forward-set-page-size)
    (define-key view-mode-map "w" 'View-scroll-page-backward-set-page-size)
    ;;    (define-key view-mode-map "b" 'View-scroll-page-backward)
    (define-key view-mode-map "\C-?" 'View-scroll-page-backward)
    ;;    (define-key view-mode-map "f" 'View-scroll-page-forward)
    (define-key view-mode-map " " 'View-scroll-page-forward)
    (define-key view-mode-map "o" 'View-scroll-to-buffer-end)
    (define-key view-mode-map ">" 'end-of-buffer)
    (define-key view-mode-map "<" 'beginning-of-buffer)
    (define-key view-mode-map "-" 'negative-argument)
    (define-key view-mode-map "9" 'digit-argument)
    (define-key view-mode-map "8" 'digit-argument)
    (define-key view-mode-map "7" 'digit-argument)
    (define-key view-mode-map "6" 'digit-argument)
    (define-key view-mode-map "5" 'digit-argument)
    (define-key view-mode-map "4" 'digit-argument)
    (define-key view-mode-map "3" 'digit-argument)
    (define-key view-mode-map "2" 'digit-argument)
    (define-key view-mode-map "1" 'digit-argument)
    (define-key view-mode-map "0" 'digit-argument)
    (define-key view-mode-map "H" 'describe-mode)
    (define-key view-mode-map "?" 'describe-mode) ; Maybe do as less instead?
    (define-key view-mode-map "h" 'describe-mode)
    ))

(provide 'viewhook20)
